from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class MSKClusterEncryption(BaseResourceCheck):
    def __init__(self):
        name = "Ensure MSK Cluster encryption in rest and transit is enabled"
        id = "CKV_AWS_81"
        supported_resources = ['aws_msk_cluster']
        categories = [CheckCategories.ENCRYPTION]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        if 'encryption_info' in conf.keys():
            encryption = conf['encryption_info'][0]
            if 'encryption_at_rest_kms_key_arn' in encryption:
                if 'encryption_in_transit' in encryption:
                    transit = encryption['encryption_in_transit'][0]
                    if 'client_broker' in transit and transit['client_broker'][0] != 'TLS' or \
                            'in_cluster' in transit and transit['in_cluster'][0] is False:
                        return CheckResult.FAILED
                    return CheckResult.PASSED
                return CheckResult.PASSED
        return CheckResult.FAILED


check = MSKClusterEncryption()



from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.arm.base_resource_check import BaseResourceCheck


class AKSRbacEnabled(BaseResourceCheck):
    def __init__(self):
        # apiVersion 2017-08-03 = Fail - No enableRBAC option to configure
        name = "Ensure RBAC is enabled on AKS clusters"
        id = "CKV_AZURE_5"
        supported_resources = ['Microsoft.ContainerService/managedClusters']
        categories = [CheckCategories.KUBERNETES]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        if "apiVersion" in conf:
            if conf["apiVersion"] == "2017-08-31":
                # No enableRBAC option to configure
                return CheckResult.FAILED

        if "properties" in conf:
            if "enableRBAC" in conf["properties"]:
                if str(conf["properties"]["enableRBAC"]).lower() == "true":
                    return CheckResult.PASSED
        return CheckResult.FAILED

check = AKSRbacEnabled()



from checkov.common.util.consts import DEFAULT_EXTERNAL_MODULES_DIR


class RunnerFilter(object):
    # NOTE: This needs to be static because different filters may be used at load time versus runtime
    #       (see note in BaseCheckRegistery.register). The concept of which checks are external is
    #       logically a "static" concept anyway, so this makes logical sense.
    __EXTERNAL_CHECK_IDS = set()

    def __init__(self, framework='all', checks=None, skip_checks=None, download_external_modules=False, external_modules_download_path=DEFAULT_EXTERNAL_MODULES_DIR, evaluate_variables=True):
        if checks is None:
            checks = []
        if isinstance(checks, str):
            self.checks = checks.split(",")
        else:
            self.checks = checks

        if skip_checks is None:
            skip_checks = []
        if isinstance(skip_checks, str):
            self.skip_checks = skip_checks.split(",")
        else:
            self.skip_checks = skip_checks
        self.framework = framework
        self.download_external_modules = download_external_modules
        self.external_modules_download_path = external_modules_download_path
        self.evaluate_variables = evaluate_variables

    def should_run_check(self, check_id):
        if RunnerFilter.is_external_check(check_id):
            pass        # enabled unless skipped
        elif self.checks:
            if check_id in self.checks:
                return True
            else:
                return False
        if self.skip_checks and check_id in self.skip_checks:
            return False
        return True

    @staticmethod
    def notify_external_check(check_id):
        RunnerFilter.__EXTERNAL_CHECK_IDS.add(check_id)

    @staticmethod
    def is_external_check(check_id):
        return check_id in RunnerFilter.__EXTERNAL_CHECK_IDS



import fnmatch
import importlib
import logging
import os
import sys
from abc import abstractmethod
from itertools import chain
from typing import Generator, Tuple

from checkov.common.checks.base_check import BaseCheck

from collections import defaultdict

from checkov.runner_filter import RunnerFilter


class BaseCheckRegistry(object):
    # NOTE: Needs to be static to because external check loading may be triggered by a registry to which
    #       checks aren't registered. (This happens with Serverless, for example.)
    __loading_external_checks = False

    def __init__(self):
        self.logger = logging.getLogger(__name__)
        # IMPLEMENTATION NOTE: Checks is used to directly access checks based on an specific entity
        self.checks = defaultdict(list)
        # IMPLEMENTATION NOTE: When using a wildcard, every pattern needs to be checked. To reduce the
        #                      number of checks checks with the same pattern are grouped, which is the
        #                      reason to use a dict for this too.
        self.wildcard_checks = defaultdict(list)
        self.check_id_allowlist = None

    def register(self, check):
        # IMPLEMENTATION NOTE: Checks are registered when the script is loaded
        #                      (see BaseResourceCheck.__init__() for the various frameworks). The only
        #                      difficultly with this process is that external checks need to be specially
        #                      identified for filter handling. That's why you'll see stateful setting of
        #                      RunnerFilters during load_external_checks.
        #                      Built-in checks are registered immediately at script start, before
        #                      external checks.
        if BaseCheckRegistry.__loading_external_checks:
            RunnerFilter.notify_external_check(check.id)

        for entity in check.supported_entities:
            checks = self.wildcard_checks if self._is_wildcard(entity) else self.checks
            checks[entity].append(check)

    @staticmethod
    def _is_wildcard(entity):
        return ('*' in entity
                or '?' in entity
                or ('[' in entity and ']' in entity))

    def get_check_by_id(self, check_id):
        return next(
            filter(
                lambda c: c.id == check_id,
                chain(*self.checks.values(), *self.wildcard_checks.values())
            ), None)

    def all_checks(self) -> Generator[Tuple[str, BaseCheck], None, None]:
        for entity, checks in self.checks.items():
            for check in checks:
                yield entity, check
        for entity, checks in self.wildcard_checks.items():
            for check in checks:
                yield entity, check

    @property
    def contains_wildcard(self) -> bool:
        return bool(self.wildcard_checks)

    def get_checks(self, entity):
        if not self.wildcard_checks:
            # Optimisation: When no wildcards are used, we can use the list in self.checks
            return self.checks.get(entity) or []
        else:
            res = self.checks[entity].copy() if entity in self.checks.keys() else []
            # check wildcards
            for pattern, checks in self.wildcard_checks.items():
                if fnmatch.fnmatchcase(entity, pattern):
                    res += checks
            return res

    def set_checks_allowlist(self, runner_filter):
        if runner_filter.checks:
            self.check_id_allowlist = runner_filter.checks

    @abstractmethod
    def extract_entity_details(self, entity):
        raise NotImplementedError()

    def scan(self, scanned_file, entity, skipped_checks, runner_filter):
        (entity_type, entity_name, entity_configuration) = self.extract_entity_details(entity)
        results = {}
        checks = self.get_checks(entity_type)
        for check in checks:
            skip_info = {}
            if skipped_checks:
                if check.id in [x['id'] for x in skipped_checks]:
                    skip_info = [x for x in skipped_checks if x['id'] == check.id][0]

            if runner_filter.should_run_check(check.id):
                result = self.run_check(check, entity_configuration, entity_name, entity_type, scanned_file, skip_info)
                results[check] = result
        return results

    def run_check(self, check, entity_configuration, entity_name, entity_type, scanned_file, skip_info):
        self.logger.debug("Running check: {} on file {}".format(check.name, scanned_file))
        result = check.run(scanned_file=scanned_file, entity_configuration=entity_configuration,
                           entity_name=entity_name, entity_type=entity_type, skip_info=skip_info)
        return result

    @staticmethod
    def _directory_has_init_py(directory):
        """ Check if a given directory contains a file named __init__.py.

        __init__.py is needed to ensure the directory is a Python module, thus
        can be imported.
        """
        if os.path.exists("{}/__init__.py".format(directory)):
            return True
        return False

    @staticmethod
    def _file_can_be_imported(entry):
        """ Verify if a directory entry is a non-magic Python file."""
        if entry.is_file() and not entry.name.startswith('__') and entry.name.endswith('.py'):
            return True
        return False

    def load_external_checks(self, directory, runner_filter):
        """ Browse a directory looking for .py files to import.

        Log an error when the directory does not contains an __init__.py or
        when a .py file has syntax error
        """
        directory = os.path.expanduser(directory)
        self.logger.debug("Loading external checks from {}".format(directory))
        sys.path.insert(1, directory)

        with os.scandir(directory) as directory_content:
            if not self._directory_has_init_py(directory):
                self.logger.info("No __init__.py found in {}. Cannot load any check here.".format(directory))
            else:
                for entry in directory_content:
                    if self._file_can_be_imported(entry):
                        check_name = entry.name.replace('.py', '')

                        # Filter is set while loading external checks so the filter can be informed
                        # of the checks, which need to be handled specially.
                        try:
                            BaseCheckRegistry.__loading_external_checks = True
                            self.logger.debug("Importing external check '{}'".format(check_name))
                            importlib.import_module(check_name)
                        except SyntaxError as e:
                            self.logger.error(
                                "Cannot load external check '{check_name}' from {check_full_path} : {error_message} ("
                                "{error_line}:{error_column}) "
                                    .format(
                                    check_name=check_name,
                                    check_full_path=e.args[1][0],
                                    error_message=e.args[0],
                                    error_line=e.args[1][1],
                                    error_column=e.args[1][2]
                                )
                            )
                        finally:
                            BaseCheckRegistry.__loading_external_checks = False



from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.arm.base_resource_check import BaseResourceCheck

# https://docs.microsoft.com/en-us/azure/templates/microsoft.web/2019-08-01/sites/config-authsettings


class AppServiceAuthentication(BaseResourceCheck):
    def __init__(self):
        name = "Ensure App Service Authentication is set on Azure App Service"
        id = "CKV_AZURE_13"
        supported_resources = ['Microsoft.Web/sites/config', 'config']
        categories = [CheckCategories.IAM]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        if "type" in conf:
            if conf["type"] == "Microsoft.Web/sites/config":
                if "name" in conf:
                    if "authsettings" in conf["name"]:
                        if "properties" in conf:
                            if "enabled" in conf["properties"]:
                                if str(conf["properties"]["enabled"]).lower() == "true":
                                    return CheckResult.PASSED
                        return CheckResult.FAILED
            elif conf["type"] == "config":
                if "name" in conf and conf["name"] == "authsettings":
                    if "parent_type" in conf:
                        if conf["parent_type"] == "Microsoft.Web/sites":
                            if "properties" in conf:
                                if "enabled" in conf["properties"]:
                                    if str(conf["properties"]["enabled"]).lower() == "true":
                                        return CheckResult.PASSED
                        return CheckResult.FAILED
        # If name not authsettings, don't return passed or failed...

check = AppServiceAuthentication()


import os
from http import HTTPStatus

import requests

from checkov.terraform.module_loading.content import ModuleContent
from checkov.terraform.module_loading.loader import ModuleLoader
from checkov.terraform.module_loading.loaders.versions_parser import order_versions_in_descending_order, get_version_constraints


class RegistryLoader(ModuleLoader):
    REGISTRY_URL_PREFIX = "https://registry.terraform.io/v1/modules"

    def __init__(self):
        super().__init__()
        self.available_versions = []

    def _is_matching_loader(self) -> bool:
        self._process_inner_registry_module()
        if os.path.exists(self.dest_dir):
            return True

        get_version_url = os.path.join(self.REGISTRY_URL_PREFIX, self.module_source, 'versions')
        response = requests.get(url=get_version_url)
        if response.status_code != HTTPStatus.OK:
            return False
        else:
            self.available_versions = [v.get('version') for v in
                                       response.json().get('modules', [{}])[0].get('versions', {})]
            return True

    def _load_module(self) -> ModuleContent:
        if os.path.exists(self.dest_dir):
            return ModuleContent(dir=None)

        best_version = self._find_best_version()

        request_download_url = os.path.join(self.REGISTRY_URL_PREFIX, self.module_source, best_version, 'download')
        response = requests.get(url=request_download_url)
        if response.status_code != HTTPStatus.OK and response.status_code != HTTPStatus.NO_CONTENT:
            return ModuleContent(dir=None)
        else:
            return ModuleContent(dir=None, next_url=response.headers.get('X-Terraform-Get', ''))

    def _find_best_version(self):
        versions_by_size = order_versions_in_descending_order(self.available_versions)
        if self.version == 'latest':
            self.version = versions_by_size[0]
        version_constraints = get_version_constraints(self.version)
        num_of_matches = 0
        for version in versions_by_size:
            for version_constraint in version_constraints:
                if not version_constraint.versions_matching(version):
                    break
                else:
                    num_of_matches += 1
            if num_of_matches == len(version_constraints):
                return version
            else:
                num_of_matches = 0
        return 'latest'

    def _process_inner_registry_module(self):
        # Check if the source has '//' in it. If it does, it indicates a reference for an inner module.
        # Example: "terraform-aws-modules/security-group/aws//modules/http-80" =>
        #    module_source = terraform-aws-modules/security-group/aws
        #    dest_dir = modules/http-80
        module_source_components = self.module_source.split('//')
        if len(module_source_components) > 1:
            self.module_source = module_source_components[0]
            self.dest_dir = self.dest_dir.split('//')[0]
            self.inner_module = module_source_components[1]


loader = RegistryLoader()



#!/usr/bin/env python
import logging
import os
from importlib import util
from os import path

import setuptools
from setuptools import setup

# read the contents of your README file
this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, "README.md"), encoding="utf-8") as f:
    long_description = f.read()

logger = logging.getLogger(__name__)
spec = util.spec_from_file_location(
    "checkov.version", os.path.join("checkov", "version.py")
)
# noinspection PyUnresolvedReferences
mod = util.module_from_spec(spec)
spec.loader.exec_module(mod)  # type: ignore
version = mod.version  # type: ignore

setup(
    extras_require={
        "dev": [
            "alabaster==0.7.12",
            "attrs==19.3.0",
            "babel==2.7.0",
            "certifi==2019.11.28",
            "chardet==3.0.4",
            "coverage==4.5.4",
            "coverage-badge==1.0.1",
            "docopt==0.6.2",
            "docutils==0.15.2",
            "idna==2.8",
            "imagesize==1.1.0",
            "importlib-metadata==1.1.0; python_version < '3.8'",
            "jinja2==2.10.3",
            "lark-parser==0.7.8",
            "markupsafe==1.1.1",
            "more-itertools==8.0.0",
            "packaging==19.2",
            "pluggy==0.13.1",
            "py==1.8.0",
            "pygments==2.5.2",
            "pyparsing==2.4.5",
            "pytest==5.3.1",
            "bc-python-hcl2>=0.3.10",
            "pytz==2019.3",
            "pyyaml==5.3.1",
            "requests==2.22.0",
            "six==1.15.0",
            "snowballstemmer==2.0.0",
            "sphinx==2.2.1",
            "sphinxcontrib-applehelp==1.0.1",
            "sphinxcontrib-devhelp==1.0.1",
            "sphinxcontrib-htmlhelp==1.0.2",
            "sphinxcontrib-jsmath==1.0.1",
            "sphinxcontrib-qthelp==1.0.2",
            "sphinxcontrib-serializinghtml==1.1.3",
            "urllib3==1.25.10",
            "wcwidth==0.1.7",
            "zipp==0.6.0",
            "GitPython==3.1.7",
            "gitdb==4.0.5",
            "packaging==20.4"
        ]
    },
    install_requires=[
        "update-checker==0.18.0",
        "tqdm==4.49.0",
        "boto3==1.12.43",
        "chardet==3.0.4",
        "colorama==0.4.3",
        "deep_merge==0.0.4",
        "docopt==0.6.2",
        "idna==2.8",
        "jmespath==0.10.0",
        "junit-xml==1.8",
        "lark-parser==0.7.8",
        "bc-python-hcl2>=0.3.11",
        "pyyaml==5.3.1",
        "requests==2.22.0",
        "six==1.15.0",
        "semantic_version==2.8.5",
        "tabulate==0.8.6",
        "termcolor==1.1.0",
        "urllib3==1.25.10",
        "dpath==1.5.0",
        "GitPython==3.1.7",
        "gitdb==4.0.5",
        "packaging==20.4"
    ],
    license="Apache License 2.0",
    name="checkov",
    version=version,
    description="Infrastructure as code static analysis",
    author="bridgecrew",
    author_email="meet@bridgecrew.io",
    url="https://github.com/bridgecrewio/checkov",
    packages=setuptools.find_packages(exclude=["tests*","integration_tests*"]),
    scripts=["bin/checkov","bin/checkov.cmd"],
    long_description=long_description,
    long_description_content_type="text/markdown",
    classifiers=[
        'Environment :: Console',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'Programming Language :: Python :: 3.7',
        'Topic :: Security',
        'Topic :: Software Development :: Build Tools'
    ]
)
